<?php
/**************************************************
  Coppermine Plugin - Sort by EXIF
  *************************************************
  Copyright (c) 2018 Ludger Hellerhoff
  *************************************************
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 3
  as published by the Free Software Foundation.
***************************************************/

if (!defined('IN_COPPERMINE')) die('Not in Coppermine...');

$cpg_base_dir = getcwd();

require_once($cpg_base_dir."/plugins/sort_by_exif/initialize.inc.php");
require_once($cpg_base_dir."/plugins/sort_by_exif/load_exif.php");
require_once($cpg_base_dir.'/plugins/sort_by_exif/qt_metadata.php');

// Add install & configure actions
$thisplugin->add_action('plugin_install','sbe_install');
$thisplugin->add_action('plugin_uninstall','sbe_uninstall');
$thisplugin->add_action('add_file_data_success','sbe_add_file_data_success');
$thisplugin->add_filter('thumb_get_pic_data_sortarray','sbe_thumb_get_pic_data_sortarray');
$thisplugin->add_filter('thumb_get_pic_pos_sortarray','sbe_thumb_get_pic_pos_sortarray');
$thisplugin->add_filter('page_html','sbe_page_html');
$thisplugin->add_filter('theme_display_thumbnails_params','sbe_theme_display_thumbnails_params');

// Install the plugin

function sbe_install() {
	global $CONFIG;
	$result = cpg_db_query("select column_name from information_schema.columns where column_name = 'sbe_exif_date'");
	$result = (cpg_db_num_rows($result) > 0);
	if ($result) {$result = True;} else {
		$result = cpg_db_query("alter table {$CONFIG['TABLE_PICTURES']} add (sbe_exif_date datetime)");
		};
	$result = cpg_db_query("select column_name from information_schema.columns where column_name = 'sbe_source_is_exif'");
	$result = (cpg_db_num_rows($result) > 0);
	if ($result) {$result = True;} else {
		$result = cpg_db_query("alter table {$CONFIG['TABLE_PICTURES']} add (sbe_source_is_exif int(1))");
		};
	$result = cpg_db_query("update {$CONFIG['TABLE_CONFIG']} set value='ea' where name = 'default_sort_order'");
	return $result;
}

function sbe_uninstall() {
	global $CONFIG;
	$result = cpg_db_query("select column_name from information_schema.columns where column_name = 'sbe_source_is_exif'");
	$result = (cpg_db_num_rows($result) == 0);
	if ($result) {$result = True;} else {
		$result = cpg_db_query("alter table {$CONFIG['TABLE_PICTURES']} drop column sbe_source_is_exif");
		};	
	$result = cpg_db_query("select column_name from information_schema.columns where column_name = 'sbe_exif_date'");
	$result = (cpg_db_num_rows($result) == 0);
	if ($result) {$result = True;} else {
		$result = cpg_db_query("alter table {$CONFIG['TABLE_PICTURES']} drop column sbe_exif_date");
		};	
	$result2 = cpg_db_query("select name, value from {$CONFIG['TABLE_CONFIG']} where (name = 'default_sort_order') and ((value = 'ea') or (value = 'ed'))");
	$result2 = (cpg_db_num_rows($result2) > 0);
	if ($result2) {
		$result = cpg_db_query("update {$CONFIG['TABLE_CONFIG']} set value='na' where name = 'default_sort_order'");	
	};
	return $result;
}

function sbe_add_file_data_success($picture) {
	var_dump($picture);
	$result = load_exif($picture);	
	return $result;
}

function sbe_thumb_get_pic_data_sortarray($sort_array) {
	$sort_array['ea'] =  str_replace('filename', 'sbe_exif_date', $sort_array['na']);
	$sort_array['ed'] =  str_replace('filename', 'sbe_exif_date', $sort_array['nd']);
	return $sort_array;
}

function sbe_thumb_get_pic_pos_sortarray($twoarrays) {
	global $CONFIG;
	$sort_array = $twoarrays[0];
	$pid = $twoarrays[1];
    $result = cpg_db_query("SELECT sbe_exif_date FROM {$CONFIG['TABLE_PICTURES']} WHERE pid = $pid");
	$pic = cpg_db_fetch_assoc($result);
	$sort_array['ea'] =  "(sbe_exif_date<'".$pic['sbe_exif_date']."' OR sbe_exif_date='".$pic["sbe_exif_date"]."' AND pid < ".$pid.")";
	$sort_array['ed'] =  "(sbe_exif_date>'".$pic['sbe_exif_date']."' OR sbe_exif_date='".$pic["sbe_exif_date"]."' AND pid > ".$pid.")";
	$twoarrays[0] = $sort_array;
	return $twoarrays;
}

function sbe_page_html($html) {
	global $lang_admin_php, $lang_plugin_sbe;
	if (ADMIN_PHP) {
		if (strpos($html, $lang_admin_php['default_sort_order'])) {
			$searchtext = '<option value="ta">'.$lang_admin_php['title_a'].'</option><option value="td">'.$lang_admin_php['title_d'].'</option>';
			$addition = '<option value="ea">'.$lang_plugin_sbe['exif_a'].'</option><option value="ed">'.$lang_plugin_sbe['exif_d'].'</option>';
			$html = str_replace($searchtext, $addition . $searchtext, $html);
		}
	}	
	return $html;
}

function sbe_theme_display_thumbnails_params($params) {
	if (GALLERY_ADMIN_MODE) {
		global $CONFIG;
		$pid = $params['{LINK_TGT}'];
		$pid = substr($pid, strpos($pid, 'displayimage.php'),200);
		$pid = substr($pid, strpos($pid, 'pid=')+4,200);
		$pid = substr($pid, 0, strpos($pid, '#top_display_media'));
		$sinfo = cpg_db_query("select sbe_exif_date, sbe_source_is_exif from {$CONFIG['TABLE_PICTURES']} WHERE pid = $pid");
		$sinfo = cpg_db_fetch_assoc($sinfo);
		$led = 'rot.png';
		if ($sinfo['sbe_source_is_exif'] == 0) {$led = 'gelb.png';};
		if (($sinfo['sbe_source_is_exif'] >= 1) and ($sinfo['sbe_source_is_exif'] <= 3)) {$led = 'gruen.png';};
		$params['{ADMIN_MENU}'] = $params['{ADMIN_MENU}'].$sinfo['sbe_exif_date'].'  '.'<img src="plugins/sort_by_exif/images/'.$led.'" width="15" height="15" border="0" alt="?" />';
	};	
	return $params;
} 

?>