<?php
/**************************************************
  Coppermine Plugin - Sort by EXIF
  *************************************************
  Copyright (c) 2018 Ludger Hellerhoff
  *************************************************
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 3
  as published by the Free Software Foundation.
***************************************************/

// if (!defined('IN_COPPERMINE')) die('Not in Coppermine...');

function return_bytes($val) {
    $val = trim($val);
    $last = strtolower($val[strlen($val)-1]);
    switch($last) {
        // The 'G' modifier is available since PHP 5.1.0
        case 'g':
			$val = substr($val,0,strlen($val)-1);
            $val *= (1024*1024*1024);
			break;
        case 'm':
			$val = substr($val,0,strlen($val)-1);
            $val *= (1024*1024);
			break;
        case 'k':
			$val = substr($val,0,strlen($val)-1);
            $val *= 1024;
			break;
    }
    return $val;
};

function BigEndian2Int($byteword) {
	$intvalue = 0;
	$bytewordlen = strlen($byteword);
	if ($bytewordlen == 0) {
		return false;
	}
	for ($i = 0; $i < $bytewordlen; $i++) {
			$intvalue += ord($byteword{$i}) * pow(256, ($bytewordlen - 1 - $i));		
	}
	return $intvalue;
};

function Int2BigEndian($intvalue) {
	$byteword = '0000';
	$byteword{3} = chr($intvalue % 256);
	$intvalue = round(floor($intvalue / 256));
	$byteword{2} = chr($intvalue % 256);
	$intvalue =  round(floor($intvalue / 256));
	$byteword{1} = chr($intvalue % 256);
	$intvalue =  round(floor($intvalue / 256));
	$byteword{0} = chr($intvalue % 256);
	$intvalue =  round(floor($intvalue / 256));
	return $byteword;
};

function QuicktimeParseContainerAtom($atom_data, $baseoffset) {
	$subatomoffset  = 0;
	$subatomcounter = 0;
	if ((strlen($atom_data) == 4) && (BigEndian2Int($atom_data) == 0x00000000)) {
		return false;
	}
	while ($subatomoffset < strlen($atom_data)) {
		$subatomsize = 	BigEndian2Int(substr($atom_data, $subatomoffset + 0, 4));
		$subatomname =  substr($atom_data, $subatomoffset + 4, 4);
		$subatomdata =  substr($atom_data, $subatomoffset + 8, intval($subatomsize - 8));
		if ($subatomsize == 0) {
			if (strlen($atom_data) > 12) {
				$subatomoffset += 4;
				continue;
			}
			return $subatoms;
		}
		if ($subatomname == 'mvhd') {		
			$subatoms['creation_time'] = BigEndian2Int(substr($subatomdata,  4, 4));
			$subatoms['modify_time'] =  BigEndian2Int(substr($subatomdata,  8, 4));
			$subatoms['c_offset'] = ($baseoffset + $subatomoffset + 12);
			$subatoms['m_offset'] = ($baseoffset + $subatomoffset + 16);	
			break;
		};
		$subatomoffset += $subatomsize;
		$subatomcounter++;
	}
	return $subatoms;
}



function has_qt_meta($input) {
	if (file_exists($input)) {
		$pattern = '^.{4}(cmov|free|ftyp|mdat|moov|pnot|skip|wide)';
		$handle = fopen($input,'r');
		if ($handle) {
			$buffer = fread($handle, 32774);
			fclose($handle);
			if ($buffer) {
				if (preg_match('#'.$pattern.'#s', $buffer)) {return 1;} else {return 0;};
			} else {return 0;}
		} else {return 0;}	
	} else {return 0;}	
};

function get_qt_meta($input) {
	$result = array();
	$php_memory_limit = return_bytes(ini_get('memory_limit'));
	if (($php_memory_limit > 0) and ($php_memory_limit < 12582912)) {
		$result['error'] = 'Not enough PHP Memory.';
	} else {	
		if (file_exists($input)) {
			$handle = fopen($input,'r');
			if ($handle) {
				$offset      = 0;
				$atomcounter = 0;	
				$file_size = filesize($input);
				$atom_buffer_size = 33554432;
				while ($offset < $file_size) {
					fseek($handle,$offset);
					$AtomHeader = fread($handle,8);
					$atomsize = BigEndian2Int(substr($AtomHeader, 0, 4));
					$atomname = substr($AtomHeader, 4, 4);
					if ($atomsize == 1) {
						$atomsize = BigEndian2Int(fread($handle,8));
					}	
					if ($atomsize == 0) {
						break;
					};
					if ($atomname == 'moov') {
						$result = QuicktimeParseContainerAtom(fread($handle,min($atomsize, $atom_buffer_size)), $offset + 8);		
						break;
					};	
					$offset += $atomsize;
					$atomcounter++;
				};
				fclose($handle);
				if (isset($result['creation_time'])) {$result['creation_time'] = date('Y-m-d H:i:s', $result['creation_time']-2082844800);};
				if (isset($result['modify_time'])) {$result['modify_time'] = date('Y-m-d H:i:s', $result['modify_time']-2082844800);};
				$result['error'] = 0;
			} else {
				$result['error'] = 'Unable to open file.';
			};
		} else {
			$result['error'] = 'File not found.';
		};
	};
	return $result;
};

function set_qt_meta($input, $creation_time, $modify_time, $coffset=NULL, $moffset=NULL) {
	if (file_exists($input)) {
		if (($coffset) and ($coffset)) {
			$temp['error'] = 0;
		} else {
			$temp = get_qt_meta($input);};  
		if ($temp['error']==0) {
			$coffset = $temp['c_offset'];
			$moffset = $temp['m_offset'];
			$c_time = strtotime($creation_time)+2082844800;
			$m_time = strtotime($modify_time)+2082844800;
			$handle = fopen($input,'r+');
			if ($handle) {
				fseek($handle,$coffset);
				fwrite($handle,Int2BigEndian($c_time));
				fseek($handle,$moffset);
				fwrite($handle,Int2BigEndian($m_time));
				fclose($handle); 
			} else {return 0;};
		};
	} else {return 0;}	
};

?>